package dk.dtu.s094758.lib.processor;

import dk.dtu.s094758.lib.model.ClassifiedGetter;
import dk.dtu.s094758.lib.model.ClassifiedVariable;
import dk.dtu.s094758.lib.model.ClassifiedSetter;
import dk.dtu.s094758.lib.annotation.Classified;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author linas
 */
public class WebServiceClassificationUtils {

    private static Boolean isGetter(Method method) {

        Boolean isGetter = false;

        if (method.getName().startsWith("get")
                && method.getParameterTypes().length == 0
                && !void.class.equals(method.getReturnType())
                && !method.getName().equals("getClass")) {

            isGetter = true;
        }

        return isGetter;
    }

    private static Boolean isSetter(Method method) {

        Boolean isSetter = false;

        if (method.getName().startsWith("set")
                && method.getParameterTypes().length == 1
                && void.class.equals(method.getReturnType())) {

            isSetter = true;
        }

        return isSetter;
    }

    private static String getVariableName(Method method) {
        return method.getName().substring(3);
    }

    public static Map<String, ClassifiedVariable> buildClassifiedVariableMap(Method[] methods) {

        Map<String, ClassifiedVariable> classifiedVariableMap = new HashMap<String, ClassifiedVariable>();

        for (Method method : methods) {

            Classified classifiedAnnotation = method.getAnnotation(Classified.class);

            if (classifiedAnnotation != null) {

                if (isSetter(method)) {

                    ClassifiedSetter classifiedSetter = new ClassifiedSetter();
                    classifiedSetter.setName(method.getName());
                    classifiedSetter.setMethod(method);
                    classifiedSetter.setClassificationLevel(classifiedAnnotation.value());

                    if (classifiedVariableMap.containsKey(getVariableName(method))) {

                        classifiedVariableMap.get(getVariableName(method)).setClassifiedSetter(classifiedSetter);

                    } else {

                        ClassifiedVariable classifiedVariable = new ClassifiedVariable();
                        classifiedVariable.setClassifiedSetter(classifiedSetter);

                        classifiedVariableMap.put(getVariableName(method), classifiedVariable);
                    }

                } else if (isGetter(method)) {

                    ClassifiedGetter classifiedGetter = new ClassifiedGetter();
                    classifiedGetter.setName(method.getName());
                    classifiedGetter.setMethod(method);
                    classifiedGetter.setClassificationLevel(classifiedAnnotation.value());

                    if (classifiedVariableMap.containsKey(getVariableName(method))) {

                        classifiedVariableMap.get(getVariableName(method)).setClassifiedGetter(classifiedGetter);

                    } else {

                        ClassifiedVariable classifiedVariable = new ClassifiedVariable();
                        classifiedVariable.setClassifiedGetter(classifiedGetter);

                        classifiedVariableMap.put(getVariableName(method), classifiedVariable);
                    }
                }
            }
        }

        return classifiedVariableMap;
    }
}
