package org.netbeans.enterprise.bpel.shopbpelmodule.shopbpelmodule;

import dk.dtu.s094758.postal.postalservice.PostalService_WSOUIServer;
import dk.dtu.s094758.company.companyservice.CompanyService_WSOUIServer;
import java.net.URL;
import java.util.List;
import java.util.Scanner;
import javax.xml.namespace.QName;
import org.petalslink.easiestdemo.wsoui.core.WSOUIException;
import org.petalslink.easiestdemo.wsoui.util.ESBUtil;
import com.ebmwebsourcing.easyesb.soa.api.node.Node;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;
import easyesb.petalslink.com.service.admin._1_0.AdminExceptionMsg;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import org.petalslink.easiestdemo.client.WSOUIClient;
import org.petalslink.easiestdemo.client.WSOUIClientException;
import java.net.MalformedURLException;
import org.petalslink.easiestdemo.wsoui.util.SOAUtil;
import org.petalslink.abslayer.service.api.Description;
import java.util.ArrayList;
import com.ebmwebsourcing.easybox.api.XmlObjectReadException;
import java.lang.Exception;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.logging.LogManager;

public class ShopBpelModule_Main {

	public final static URL BPEL_LOCATION;
	static {
	    BPEL_LOCATION = Thread.currentThread().getContextClassLoader().getResource("ShopBpelModule.bpel");
	    System.out.println("************************ BPEL_LOCATION = " + BPEL_LOCATION);
	}

	static {
		try {
			InputStream configFile = Thread.currentThread().getContextClassLoader().getResourceAsStream("logger/easiestdemo-logging.properties");
			LogManager.getLogManager().readConfiguration(configFile);
		} catch (Exception ex)
		{
			System.out.println("WARNING: Could not open configuration file");
			System.out.println("WARNING: Logging not configured (console output only)");
		}
	}
	private PostalService_WSOUIServer postalService_WSOUIServer = null;
	private CompanyService_WSOUIServer companyService_WSOUIServer = null;

	private Node node = null;
	private QName nodeName = new QName("http://enterprise.netbeans.org/bpel/ShopBpelModule/shopBpelModule", "ShopBpelModule_EasyESBNode");
	private String nodeHost = "localhost";
	private int nodeInternalPort = 8000;
	private int nodeSoapPort = 8085;
	private String adminUrl = "http://" + nodeHost + ":" + nodeSoapPort + "/services/adminExternalEndpoint";

	private static boolean standalone = false;


	public static void main(String args[]) throws Exception {
		ShopBpelModule_Main main = new ShopBpelModule_Main();
		PrintStream err = System.err;
		try {
		
			boolean activeGUI = true;
			analyzeCommand(args);
		
			// Create and Start all WebServices
			main.startAllWebServices();
		
			// Set GUI on all WebServices
			main.setGUIOnAllWebServices(activeGUI);
		
			// Create EasyESB node
			main.createNode();
		
			// Deploy and Expose BPEL
			List<URL> bpelUrls = main.deployAndExposeBPEL(ShopBpelModule_Main.BPEL_LOCATION);
		
			// Create and Start client
			if(activeGUI) {
				main.createClient();
			}
			// Exit
			Scanner sc = new Scanner(System.in);
			System.out.println("Hit Enter to exit :");
			sc.nextLine();
		
		} catch(Exception e) {
		
			e.printStackTrace(err);
		
			// Stop all WebServices
			main.stopAllWebServices();
		
			// Stop node
			main.stopNode();
		
		}
	}


	public static void analyzeCommand(String args[]) {
		System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!! ARGS = " + args);
		if(args.length == 1 && args[0].equals("standalone")) {
			standalone = true;
		}
	}


	public boolean isStandalone() {
		return standalone;
	}

	public void createNode() throws ESBException {
		this.node = ESBUtil.createNode(nodeName, nodeHost, nodeInternalPort, nodeSoapPort);
	}


	public List<URL> deployAndExposeBPEL(URL bpel) throws MalformedURLException, AdminExceptionMsg {
		List<URL> urls = ESBUtil.deployAndExposeBPEL(bpel, new URL(adminUrl));

		System.out.println("BPEL " + bpel + " are exposed at url:");
		for(URL url: urls) {
			System.out.println("	" + url);
		}
		return urls;
	}


	public void stopNode() throws TransportException {
		this.node.stop();
	}


	public void startAllWebServices() throws WSOUIException {
		if(!isStandalone()) {
			postalService_WSOUIServer = new PostalService_WSOUIServer();
			postalService_WSOUIServer.start();
			companyService_WSOUIServer = new CompanyService_WSOUIServer();
			companyService_WSOUIServer.start();
		}
	}


	public void createClient() throws WSOUIException {
		try {
			WSOUIClient client = new WSOUIClient();
			org.petalslink.easiestdemo.client.model.api.esb.Node n = client.addNewAdminWsdlofESBNode(new URL(adminUrl + "?wsdl"));
			n.setModel(node);
			client.setVisible(true);
		} catch (MalformedURLException e) {
			throw new WSOUIException(e);
		} catch (WSOUIClientException e) {
			throw new WSOUIException(e);
		}
	}


	public void setGUIOnAllWebServices(boolean activeGUI) throws WSOUIException {
		if(!isStandalone()) {
		if(postalService_WSOUIServer != null) {
			postalService_WSOUIServer.getProviderConfigurationFor_PostalService().setActiveGui(activeGUI);
		}
		if(companyService_WSOUIServer != null) {
			companyService_WSOUIServer.getProviderConfigurationFor_CompanyService().setActiveGui(activeGUI);
		}
		}
	}


	public void stopAllWebServices() {
		if(!isStandalone()) {
		if(postalService_WSOUIServer != null) {
			postalService_WSOUIServer.stop();
		}
		if(companyService_WSOUIServer != null) {
			companyService_WSOUIServer.stop();
		}
		}
	}



}
