/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package dk.dtu.s094758.generator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.ode.bpel.compiler.bom.Activity;
import org.apache.ode.bpel.compiler.bom.AssignActivity;
import org.apache.ode.bpel.compiler.bom.BpelObjectFactory;
import org.apache.ode.bpel.compiler.bom.IfActivity;
import org.apache.ode.bpel.compiler.bom.InvokeActivity;
import org.apache.ode.bpel.compiler.bom.Process;
import org.apache.ode.bpel.compiler.bom.ReceiveActivity;
import org.apache.ode.bpel.compiler.bom.ReplyActivity;
import org.apache.ode.bpel.compiler.bom.SequenceActivity;
import org.apache.ode.bpel.compiler.bom.Variable;
import org.apache.ode.bpel.compiler.bom.Variable.Kind;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 *
 * @author linas
 */
public class BomGenerator {

    private static Map<String, String[]> messageTypeToJavaTypeMap;

    private static void parse() {

        StringBuilder stringBuilder = new StringBuilder();

        String fileLocation = "./src/main/bpel/ShopBpelModule.bpel";
        File file = new File(fileLocation);

        URI uri = null;
        try {
            uri = new URI(fileLocation);
        } catch (URISyntaxException ex) {
            Logger.getLogger(BomGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }

        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(file);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(BomGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }

        Reader reader = null;
        try {
            reader = new InputStreamReader(inputStream, "UTF-8");
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(BomGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }

        InputSource is = new InputSource(reader);

        BpelObjectFactory bpelObjectFactory = new BpelObjectFactory();

        Process bpelProcess = null;
        try {
            bpelProcess = bpelObjectFactory.parse(is, uri);
        } catch (IOException ex) {
            Logger.getLogger(BomGenerator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXException ex) {
            Logger.getLogger(BomGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }

        List<Variable> variables = bpelProcess.getVariables();

        for (Variable variable : variables) {
            Kind varKind = variable.getKind();
            String varName = variable.getName();
            String varTypeName = variable.getTypeName().getLocalPart();

            String[] javaTypes = messageTypeToJavaTypes(varTypeName);

            for (String javaType : javaTypes) {
                System.out.println(javaType);
            }

            Node jifLableNode = variable.getElement().getFirstChild();

            if (jifLableNode != null) {
                stringBuilder.append("{");
                stringBuilder.append(jifLableNode.getTextContent());
                stringBuilder.append("}");
            }


            System.out.println(varName + " " + varKind.name() + " " + varTypeName);
        }


        Activity rootActivity = bpelProcess.getRootActivity();

        parseActivity(rootActivity);
    }

    private static void parseActivity(Activity act) {

        if (act.getType().getLocalPart().equals("sequence")) {
            System.out.println("found a sequence");
            SequenceActivity seq = ((SequenceActivity) act);
            for (Activity for_act : seq.getActivities()) {
                parseActivity(for_act);
            }

        } else if (act instanceof InvokeActivity) {
            System.out.println("found an invoke");
            InvokeActivity invoke = ((InvokeActivity) act);
            System.out.println("parteneLink invoved "
                    + invoke.getPartnerLink());
            System.out.println("operation " + invoke.getOperation());
            System.out.println();

        } else if (act instanceof ReceiveActivity) {
            System.out.println("found an receive");
            ReceiveActivity receive = ((ReceiveActivity) act);
            System.out.println("parteneLink invoved "
                    + receive.getPartnerLink());
            System.out.println("operation " + receive.getOperation());
            System.out.println();

        } else if (act instanceof ReplyActivity) {
            System.out.println("found an replay");
            ReplyActivity reply = ((ReplyActivity) act);
            System.out.println("parteneLink invoved "
                    + reply.getPartnerLink());
            System.out.println("operation " + reply.getOperation());
            System.out.println();

        } else if (act instanceof AssignActivity) {
            System.out.println("found an assign");
            AssignActivity assignActivity = ((AssignActivity) act);

        } else if (act instanceof IfActivity) {
            System.out.println("found an if");
            IfActivity ifActivity = ((IfActivity) act);
        }
    }

    private static String[] messageTypeToJavaTypes(String messageType) {

        if (messageTypeToJavaTypeMap == null) {

            messageTypeToJavaTypeMap = new HashMap<String, String[]>();

            messageTypeToJavaTypeMap.put("buyProductResponse", new String[]{"boolean"});
            messageTypeToJavaTypeMap.put("buyProduct", new String[]{"dk.dtu.s094758.company.companyservice.Product"});
            messageTypeToJavaTypeMap.put("validateAddressResponse", new String[]{"boolean"});
            messageTypeToJavaTypeMap.put("validateAddress", new String[]{"dk.dtu.s094758.postal.postalservice.Address", "dk.dtu.s094758.postal.postalservice.Person"});
            messageTypeToJavaTypeMap.put("validateCustomerResponse", new String[]{"boolean"});
            messageTypeToJavaTypeMap.put("validateCustomer", new String[]{"dk.dtu.s094758.company.companyservice.Customer"});
            messageTypeToJavaTypeMap.put("orderProductResponse", new String[]{"dk.dtu.s094758.shop.shopservice.Client", "dk.dtu.s094758.shop.shopservice.Order"});
            messageTypeToJavaTypeMap.put("orderProduct", new String[]{"dk.dtu.s094758.shop.shopservice.Receipt"});
        }

        return messageTypeToJavaTypeMap.get(messageType);
    }

    public static void main(String argv[]) {

        parse();
    }
}
