package dk.dtu.ws1.filter;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

/**
 *
 * @author linas
 */
public class SecurityServletResponseWrapper extends HttpServletResponseWrapper {

    private SecurityServletOutputStream securityServletOutputStream = new SecurityServletOutputStream();
    private PrintWriter printWriter = null;
    private ServletOutputStream servletOutputStream = null;

    public SecurityServletResponseWrapper(HttpServletResponse response) {
        super(response);
    }

    public byte[] getBuffer() {
        return securityServletOutputStream.getBuffer();
    }

    @Override
    public PrintWriter getWriter() throws IOException {

        if (servletOutputStream != null) {
            throw new IllegalStateException(
                    "The Servlet API forbids calling getWriter( ) after"
                    + " getOutputStream( ) has been called");
        }

        if (printWriter == null) {
            printWriter = new PrintWriter(securityServletOutputStream);
        }

        return printWriter;
    }

    @Override
    public ServletOutputStream getOutputStream() throws IOException {
        if (printWriter != null) {
            throw new IllegalStateException(
                    "The Servlet API forbids calling getOutputStream( ) after"
                    + " getWriter( ) has been called");
        }

        if (servletOutputStream == null) {
            servletOutputStream = securityServletOutputStream;
        }

        return servletOutputStream;
    }

    @Override
    public void flushBuffer() throws IOException {

        if (servletOutputStream != null) {
            servletOutputStream.flush();
        } else if (printWriter != null) {
            printWriter.flush();
        }
    }

    @Override
    public int getBufferSize() {
        return securityServletOutputStream.getBuffer().length;
    }

    @Override
    public void reset() {
        securityServletOutputStream.reset();
    }

    @Override
    public void resetBuffer() {
        securityServletOutputStream.reset();
    }

    @Override
    public void setBufferSize(int size) {
        securityServletOutputStream.setBufferSize(size);
    }
}
